@@ -15,19 +15,22 @@ package template
 
 import (
 	"bytes"
+	tmplhtml "html/template"
+	"io"
 	"net/url"
+	"path"
 	"path/filepath"
 	"regexp"
 	"sort"
 	"strings"
+	tmpltext "text/template"
 	"time"
 
-	tmplhtml "html/template"
-	tmpltext "text/template"
-
 	"github.com/prometheus/common/model"
+	"golang.org/x/text/cases"
+	"golang.org/x/text/language"
 
-	"github.com/prometheus/alertmanager/template/internal/deftmpl"
+	"github.com/prometheus/alertmanager/asset"
 	"github.com/prometheus/alertmanager/types"
 )
 
@@ -39,48 +42,93 @@ type Template struct {
 	ExternalURL *url.URL
 }
 
-// FromGlobs calls ParseGlob on all path globs provided and returns the
-// resulting Template.
-func FromGlobs(paths ...string) (*Template, error) {
+// Option is generic modifier of the text and html templates used by a Template.
+type Option func(text *tmpltext.Template, html *tmplhtml.Template)
+
+// New returns a new Template with the DefaultFuncs added. The DefaultFuncs
+// have precedence over any added custom functions. Options allow customization
+// of the text and html templates in given order.
+func New(options ...Option) (*Template, error) {
 	t := &Template{
 		text: tmpltext.New("").Option("missingkey=zero"),
 		html: tmplhtml.New("").Option("missingkey=zero"),
 	}
-	var err error
 
-	t.text = t.text.Funcs(tmpltext.FuncMap(DefaultFuncs))
-	t.html = t.html.Funcs(tmplhtml.FuncMap(DefaultFuncs))
+	for _, o := range options {
+		o(t.text, t.html)
+	}
 
-	b, err := deftmpl.Asset("template/default.tmpl")
+	t.text.Funcs(tmpltext.FuncMap(DefaultFuncs))
+	t.html.Funcs(tmplhtml.FuncMap(DefaultFuncs))
+
+	return t, nil
+}
+
+// FromGlobs calls ParseGlob on all path globs provided and returns the
+// resulting Template.
+func FromGlobs(paths []string, options ...Option) (*Template, error) {
+	t, err := New(options...)
 	if err != nil {
 		return nil, err
 	}
-	if t.text, err = t.text.Parse(string(b)); err != nil {
-		return nil, err
-	}
-	if t.html, err = t.html.Parse(string(b)); err != nil {
-		return nil, err
-	}
 
-	for _, tp := range paths {
-		// ParseGlob in the template packages errors if not at least one file is
-		// matched. We want to allow empty matches that may be populated later on.
-		p, err := filepath.Glob(tp)
+	defaultTemplates := []string{"default.tmpl", "email.tmpl"}
+
+	for _, file := range defaultTemplates {
+		f, err := asset.Assets.Open(path.Join("/templates", file))
 		if err != nil {
 			return nil, err
 		}
-		if len(p) > 0 {
-			if t.text, err = t.text.ParseGlob(tp); err != nil {
-				return nil, err
-			}
-			if t.html, err = t.html.ParseGlob(tp); err != nil {
-				return nil, err
-			}
+		if err := t.Parse(f); err != nil {
+			f.Close()
+			return nil, err
+		}
+		f.Close()
+	}
+
+	for _, tp := range paths {
+		if err := t.FromGlob(tp); err != nil {
+			return nil, err
 		}
 	}
 	return t, nil
 }
 
+// Parse parses the given text into the template.
+func (t *Template) Parse(r io.Reader) error {
+	b, err := io.ReadAll(r)
+	if err != nil {
+		return err
+	}
+	if t.text, err = t.text.Parse(string(b)); err != nil {
+		return err
+	}
+	if t.html, err = t.html.Parse(string(b)); err != nil {
+		return err
+	}
+	return nil
+}
+
+// FromGlob calls ParseGlob on given path glob provided and parses into the
+// template.
+func (t *Template) FromGlob(path string) error {
+	// ParseGlob in the template packages errors if not at least one file is
+	// matched. We want to allow empty matches that may be populated later on.
+	p, err := filepath.Glob(path)
+	if err != nil {
+		return err
+	}
+	if len(p) > 0 {
+		if t.text, err = t.text.ParseGlob(path); err != nil {
+			return err
+		}
+		if t.html, err = t.html.ParseGlob(path); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 // ExecuteTextString needs a meaningful doc comment (TODO(fabxc)).
 func (t *Template) ExecuteTextString(text string, data interface{}) (string, error) {
 	if text == "" {
@@ -122,12 +170,18 @@ type FuncMap map[string]interface{}
 var DefaultFuncs = FuncMap{
 	"toUpper": strings.ToUpper,
 	"toLower": strings.ToLower,
-	"title":   strings.Title,
+	"title": func(text string) string {
+		// Casers should not be shared between goroutines, instead
+		// create a new caser each time this function is called.
+		return cases.Title(language.AmericanEnglish).String(text)
+	},
+	"trimSpace": strings.TrimSpace,
 	// join is equal to strings.Join but inverts the argument order
 	// for easier pipelining in templates.
 	"join": func(sep string, s []string) string {
 		return strings.Join(s, sep)
 	},
+	"match": regexp.MatchString,
 	"safeHtml": func(text string) tmplhtml.HTML {
 		return tmplhtml.HTML(text)
 	},
@@ -135,6 +189,9 @@ var DefaultFuncs = FuncMap{
 		re := regexp.MustCompile(pattern)
 		return re.ReplaceAllString(text, repl)
 	},
+	"stringSlice": func(s ...string) []string {
+		return s
+	},
 }
 
 // Pair is a key/value string pair.
@@ -163,6 +220,19 @@ func (ps Pairs) Values() []string {
 	return vs
 }
 
+func (ps Pairs) String() string {
+	b := strings.Builder{}
+	for i, p := range ps {
+		b.WriteString(p.Name)
+		b.WriteRune('=')
+		b.WriteString(p.Value)
+		if i < len(ps)-1 {
+			b.WriteString(", ")
+		}
+	}
+	return b.String()
+}
+
 // KV is a set of key/value string pairs.
 type KV map[string]string
 
@@ -215,6 +285,10 @@ func (kv KV) Values() []string {
 	return kv.SortedPairs().Values()
 }
 
+func (kv KV) String() string {
+	return kv.SortedPairs().String()
+}
+
 // Data is the data passed to notification templates and webhook pushes.
 //
 // End-users should not be exposed to Go's type system, as this will confuse them and prevent
@@ -239,6 +313,7 @@ type Alert struct {
 	StartsAt     time.Time `json:"startsAt"`
 	EndsAt       time.Time `json:"endsAt"`
 	GeneratorURL string    `json:"generatorURL"`
+	Fingerprint  string    `json:"fingerprint"`
 }
 
 // Alerts is a list of Alert objects.
@@ -269,7 +344,7 @@ func (as Alerts) Resolved() []Alert {
 // Data assembles data for template expansion.
 func (t *Template) Data(recv string, groupLabels model.LabelSet, alerts ...*types.Alert) *Data {
 	data := &Data{
-		Receiver:          strings.SplitN(recv, "/", 2)[0],
+		Receiver:          regexp.QuoteMeta(recv),
 		Status:            string(types.Alerts(alerts...).Status()),
 		Alerts:            make(Alerts, 0, len(alerts)),
 		GroupLabels:       KV{},
@@ -288,6 +363,7 @@ func (t *Template) Data(recv string, groupLabels model.LabelSet, alerts ...*type
 			StartsAt:     a.StartsAt,
 			EndsAt:       a.EndsAt,
 			GeneratorURL: a.GeneratorURL,
+			Fingerprint:  a.Fingerprint().String(),
 		}
 		for k, v := range a.Labels {
 			alert.Labels[string(k)] = string(v)
@@ -308,6 +384,9 @@ func (t *Template) Data(recv string, groupLabels model.LabelSet, alerts ...*type
 			commonAnnotations = alerts[0].Annotations.Clone()
 		)
 		for _, a := range alerts[1:] {
+			if len(commonLabels) == 0 && len(commonAnnotations) == 0 {
+				break
+			}
 			for ln, lv := range commonLabels {
 				if a.Labels[ln] != lv {
 					delete(commonLabels, ln)
